<?php

/*	LibFox - Simple library for XML-based backends.
	Copyright © 2007, 2008 LinFox Serviços em Informatica LTDA.

	This file is part of LibFox.

	LibFox is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	LibFox is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with LibFox.  If not, see <http://www.gnu.org/licenses/>. */

/**
 * boleto_dao
 * 
 * @package boleto
 */
class bill_dao extends dao
{
	public $key = 'invoice_id';
	public $foreign_keys = 'client_id,method_id';
//	public $other_keys = 'payments.client_id,payments.method_id';
	protected $table = 'invoices';
	protected $fields = 'client_id,method_id,date';
	protected $fields_all = 'client_id,method_id,date';
	protected $fields_rel_all = array(
		'client_username' => array('clients' => 'username'),
		'client_name' => array('clients' => 'name'),
		'client_address1' => array('clients' => 'address1'),
		'client_address2' => array('clients' => 'address2'),
		'client_district' => array('clients' => 'district'),
		'client_postal_code' => array('clients' => 'postal_code'),
		'client_state' => array('clients' => 'state'),
		'account_bank_code' => array('accounts' => 'bank_code'),
		'account_agency' => array('accounts' => 'agency'),
		'account_number' => array('accounts' => 'account_number'),
		'account_owner_name' => array('accounts' => 'owner_name'),
		'account_owner_cpf_cnpj' => array('accounts' => 'owner_cpf_cnpj'),
		'account_owner_address' => array('accounts' => 'owner_address'),
		'account_owner_city' => array('accounts' => 'owner_city'),
		'account_owner_state' => array('accounts' => 'owner_state'),
		'account_payment_number' => array('accounts' => 'payment_number'),
		'account_payment_info1' => array('accounts' => 'payment_info1'),
		'account_payment_info2' => array('accounts' => 'payment_info2'),
		'account_payment_info3' => array('accounts' => 'payment_info3'),
		'account_payment_info4' => array('accounts' => 'payment_info4'),
		'account_payment_instructions1' => array('accounts' => 'payment_instructions1'),
		'account_payment_instructions2' => array('accounts' => 'payment_instructions2'),
		'account_payment_instructions3' => array('accounts' => 'payment_instructions3'),
		'account_payment_instructions4' => array('accounts' => 'payment_instructions4'),
		'account_currency' => array('accounts' => 'currency')
	);
	protected $fields_sum = array(
		'paid' => array('payments' => 'value'),
		'value' => array('invoices' => 'value'),
		'discount' => array('invoices' => 'discount') );
	protected $select_sql = 'SUM(invoices.value) - SUM(payments.value) AS net_value';
	protected $left_joins = array(
		'contracts.contract_id' => 'invoices.contract_id',
		'clients.client_id' => 'invoices.client_id',
		'services.service_id' => 'invoices.service_id',
		'payment_methods.method_id' => 'invoices.method_id',
		'accounts.account_id' => 'payment_methods.account_id' );
	protected $joins_sql = '
		LEFT JOIN payments ON (
			payments.invoice_id = invoices.invoice_id
			AND payments.status != \'C\' )';
	protected $where = 'invoices.status != \'C\'';
	public $group_by = 'client_id,method_id';

	public function __construct()
	{
		parent::__construct();
	}

	public function fetch_all($model = NULL)
	{
		if (!is_object($model)) {
			throw new Exception("bill_dao::fetch_all expects an object as parameter");
			return false;
		}
		$models = array();
		$sql = "
			SELECT
				invoices.client_id,
				invoices.method_id,
				invoices.date,
				SUM(invoices.paid) AS paid,
				SUM(invoices.value) AS value,
				SUM(invoices.discount) AS discount,
				clients.username AS client_username,
				clients.name AS client_name,
				clients.address1 AS client_address1,
				clients.address2 AS client_address2,
				clients.district AS client_district,
				clients.postal_code AS client_postal_code,
				clients.state AS client_state,
				accounts.bank_code AS account_bank_code,
				accounts.agency AS account_agency,
				accounts.account_number AS account_number,
				accounts.owner_name AS account_owner_name,
				accounts.owner_cpf_cnpj AS account_owner_cpf_cnpj,
				accounts.owner_address AS account_owner_address,
				accounts.owner_city AS account_owner_city,
				accounts.owner_state AS account_owner_state,
				accounts.payment_number AS account_payment_number,
				accounts.payment_info1 AS account_payment_info1,
				accounts.payment_info2 AS account_payment_info2,
				accounts.payment_info3 AS account_payment_info3,
				accounts.payment_info4 AS account_payment_info4,
				accounts.payment_instructions1 AS account_payment_instructions1,
				accounts.payment_instructions2 AS account_payment_instructions2,
				accounts.payment_instructions3 AS account_payment_instructions3,
				accounts.payment_instructions4 AS account_payment_instructions4,
				accounts.currency AS account_currency,
				COUNT(*) AS count,
				SUM(invoices.value) - SUM(invoices.paid) AS net_value,
				invoices.payment_status AS payment_status
			FROM (
			
				SELECT payments.payment_id,
					payments.value AS paid,
					payments.status AS payment_status,
					invoices.invoice_id,
					COALESCE(invoices.client_id,payments.client_id) AS client_id,
					COALESCE(invoices.method_id,payments.method_id) AS method_id,
					COALESCE(invoices.contract_id,payments.contract_id) AS contract_id,
					invoices.service_id,
					invoices.value,
					invoices.discount,
					invoices.date,
					invoices.status
				FROM invoices
				LEFT JOIN payments ON (
					payments.invoice_id = invoices.invoice_id
					AND payments.method_id = invoices.method_id )
				WHERE (invoices.client_id = '$model->client_id' OR payments.client_id = '$model->client_id')
					AND (payments.status IS NULL OR payments.status != 'C')
			
				UNION
			
				SELECT payments.payment_id,
					payments.value AS paid,
					payments.status AS payment_status,
					invoices.invoice_id,
					COALESCE(invoices.client_id,payments.client_id) AS client_id,
					COALESCE(invoices.method_id,payments.method_id) AS method_id,
					COALESCE(invoices.contract_id,payments.contract_id) AS contract_id,
					invoices.service_id,
					invoices.value,
					invoices.discount,
					invoices.date,
					invoices.status
				FROM invoices
				RIGHT JOIN payments ON (
					payments.invoice_id = invoices.invoice_id
					AND payments.method_id = invoices.method_id )
				WHERE (invoices.client_id = '$model->client_id' OR payments.client_id = '$model->client_id')
					AND (payments.status IS NULL OR payments.status != 'C')
			
				) AS invoices
			LEFT JOIN clients ON (
				clients.client_id = invoices.client_id )
			LEFT JOIN payment_methods ON (
				payment_methods.method_id = invoices.method_id )
			LEFT JOIN accounts ON (
				accounts.account_id = payment_methods.account_id )
			WHERE (invoices.status IS NULL OR invoices.status != 'C')
			GROUP BY client_id, method_id";
		$result = $this->db->query($sql);
		$result->setFetchMode(PDO::FETCH_CLASS,$this->model);
		return $result->fetchAll();
	}
	
}

?>