<?php

/*	Vulpes - Gateway Access Manager
	Copyright © 2007, 2008 LinFox Serviços em Informatica LTDA.

	This file is part of Vulpes.

	Vulpes is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Vulpes is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Vulpes.  If not, see <http://www.gnu.org/licenses/>. */

require_once 'mdl/client.php';

/**
 * Client Data Access Object
 * @author Alwin Garside <alwin@linfox.com.br>
 * @version 0.3
 * @package client
 */
class client_dao extends dao
{
	public $key = 'client_id';
	protected $model = 'client';
	protected $table = 'clients';
	protected $fields_all = 'client_id,username,name,email';
	protected $joins = array(
		'client_accounts.client_id' => 'clients.client_id',
		'contracts.client_id' => 'clients.client_id',
		'services.service_id' => 'contracts.service_id' );
	protected $search_fields = 'clients.username,clients.name,clients.email,services.name,client_accounts.ipv4';
	protected $delete_rel = 'client_sessions,contracts';
	protected $hash = 'sha512';
	public $order_by = 'name,username';
	public $group_by = 'clients.client_id';
	
	/**
	 * Constructor for client_dao
	 *
	 * @param mysqli $db mysqli connection
	 */
	public function __construct()
	{
		parent::__construct();
	}

	public function fetch_pending($client_id)
	{
		$sql = 'SELECT payments.*
			FROM payments
			WHERE client_id = \''.addslashes($client_id).'\'';
		$result = $this->db->query($sql);

		$sql = '
			SELECT contracts.*,
				services.price
			FROM contracts
			INNER JOIN clients ON (
				clients.client_id = contracts.client_id )
			INNER JOIN services ON (
				services.service_id = contracts.service_id )
			WHERE services.price > 0
				AND clients.client_id = \''.addslashes($client_id).'\'';
		$result = $this->db->query($sql);
		$result->setFetchMode(PDO::FETCH_CLASS,'model');
		$contracts = $result->fetchAll();

		foreach ($contracts as $contract) {
			if ($contract->payment > 0) {
				$months = array();
				$month = substr($contract->start,5,2);
				$year = substr($contract->start,0,4);
				$cur_month = date('m');
				$cur_year = date('Y');

				while ($year <= $cur_year || $month <= $cur_month) {
					$months[$year.'-'.$month] = '';
					if ($month < 12) {
						$month++;
					} else {
						$month = 1;
						$year++;
					}
				}

				$diff_year = $cur_year - $start_year;
				if (($diff) > 0)
					$months += ($diff_year-1)*12;
				$diff_month = $cur_month - $start_month;
				if ($cur_month >= $start_month)
					$months += $diff_month;
				else
					$months += 12+$diff_month;
				$contract->value = ($months/$contract->payment)*$contract->price;
			}
		}
		return $contracts;
	}

	public function update_password(&$client,$password)
	{
		$sql_update_password = '
			UPDATE clients
			SET password = \''.hash('sha512', $password).'\'
			WHERE '.$this->key.' = \''.addslashes($client->client_id).'\'';
		if (!$this->db->query($sql_update_password)) {
			throw new Exception('password_not_saved');
			return false;
		}
		return true;
	}
	
	public function verify_password(&$client,$password1)
	{
		$sql_password2 = '
			SELECT password
			FROM '.$this->table.'
			WHERE '.$this->key.' = \''.addslashes($client->client_id).'\'';
		if (!$result_password2 = $this->db->query($sql_password2)) {
			throw new Exception('password_query_failed');
			return false;
		} elseif (!$result_password2->num_rows) {
			$result_password2->closeCursor();
			throw new Exception('client_or_password_incorrect');
			return false;
		} 
		$password2 = $result_password2->fetchObject();
		$result_password->closeCursor();
		if ($password2->password != $password1) {
			return false;
		} else {
			return true;
		}
	}

	/**
	 * Authenticate a client
	 * 
	 * em caso que nao, tenta autenticar usando os valores no $_POST
	 * em caso que o usuario é autenticado, os dados sao guardados nas variaveis desse class
	 */
	public function auth()
	{
		if ($_POST['action'] == 'logout' || $_GET['action'] == 'logout') {
			$_SESSION = array();
			session_destroy();
			return false;
		} elseif ($_POST['lf_client_username'] && isset($_POST['lf_client_password'])) {
			$sql_client = '
				SELECT *
				FROM '.$this->table.'
				WHERE username = \''.addslashes(get('client_username')).'\'
					AND password = \''.hash($this->hash, $_POST['lf_client_password']).'\'';
			$result = $this->db->query($sql_client);

			$client = $result->fetchObject($this->model);
			$result->closeCursor();
			if ($client) {
				$_SESSION['vulpes_client_session'] = true;
				$_SESSION['vulpes_client_username'] = $client->username;
				return $client;
			} else {
				return false;
			}
		} elseif ($_SESSION['vulpes_client_session'] == true) {
			$sql_client = '
				SELECT *
				FROM '.$this->table.'
				WHERE username = \''.$_SESSION['vulpes_client_username'].'\'';
			$result = $this->db->query($sql_client);
			$client = $result->fetchObject($this->model);
			$result->closeCursor();
			if ($client) {
				$result->closeCursor();
				return $client;
			}
		}
		return false;
	}
}

?>
