<?php

/*	Vulpes - Gateway Access Manager
	Copyright © 2007, 2008 LinFox Serviços em Informatica LTDA.

	This file is part of Vulpes.

	Vulpes is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Vulpes is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Vulpes.  If not, see <http://www.gnu.org/licenses/>. */

class invoice_dao extends dao
{
	public $key = 'invoice_id';
	public $foreign_keys = 'client_id,contract_id,service_id,sale_id,method_id';
	protected $table = 'invoices';
	protected $fields_rel_all = array(
		'client_username' => array('clients' => 'username'),
		'client_name' => array('clients' => 'name'),
		'contract_status' => array('contracts' => 'status'),
		'service_name' => array('services' => 'name') );
	protected $fields_sum = array(
		'paid' => array('payments' => 'value') );
	protected $select_sql = '
		SUM(invoices.value) - SUM(payments.value) AS net_value';
	protected $joins = array(
		'contracts.contract_id' => 'invoices.contract_id',
		'clients.client_id' => 'invoices.client_id',
		'services.service_id' => 'contracts.service_id' );
	protected $joins_sql = '
		LEFT JOIN payments ON (
			payments.invoice_id = invoices.invoice_id
			AND payments.status <> \'C\' )';
	protected $search_fields = 'invoices.invoice_id,invoices.date,invoices.value,services.name,clients.username,clients.name';
	protected $where = 'invoices.status <> \'C\'';
	public $group_by = 'invoice_id';
	public $order_by = 'date-';
	
	function __construct()
	{
		parent::__construct();
	}

	public function fetch_paid($search_model = false)
	{
		$sql = $this->sql_select_all()
			.$this->sql_from()
			.$this->sql_joins('INNER')
			.$this->sql_where($search_model)
			.$this->sql_group_by()
			.' HAVING paid >= invoices.value'
			.$this->sql_order_by();
		$result = $this->db->query($sql);
		$result->setFetchMode(PDO::FETCH_CLASS,$this->model);
		return $result->fetchAll();
	}

	public function fetch_pending($search_model = false)
	{
		$sql = $this->sql_select_all()
			.$this->sql_from()
			.$this->sql_joins('INNER')
			.$this->sql_where($search_model)
			.$this->sql_group_by()
			.' HAVING paid < invoices.value OR paid IS NULL'
			.$this->sql_order_by();
		$result = $this->db->query($sql);
		$result->setFetchMode(PDO::FETCH_CLASS,$this->model);
		return $result->fetchAll();
	}

	public function fetch_canceled($search_model = false)
	{
		$sql  = $this->sql_select_all()
			.$this->sql_from()
			.$this->sql_joins('INNER')
			.$this->sql_where($search_model,'invoices.status = \'C\'')
			.$this->sql_group_by()
			.$this->sql_order_by();
		$result = $this->db->query($sql);
		$result->setFetchMode(PDO::FETCH_CLASS,$this->model);
		return $result->fetchAll();
	}

	public function fetch_paid_like($search_model = false, $search = false)
	{
		$models = array();
		$sql = $this->sql_select_all()
			.$this->sql_from()
			.$this->sql_joins('INNER')
			.$this->sql_where_like($search_model, $search, false)
			.$this->sql_group_by()
			.' HAVING paid >= invoices.value'
			.$this->sql_order_by();

		$result = $this->db->query($sql);
		$result->setFetchMode(PDO::FETCH_CLASS,$this->model);
		return $result->fetchAll();
	}

	public function fetch_pending_like($search_model = false, $search = false)
	{
		$models = array();
		$sql = $this->sql_select_all()
			.$this->sql_from()
			.$this->sql_joins('INNER')
			.$this->sql_where_like($search_model, $search, false)
			.$this->sql_group_by()
			.' HAVING paid < invoices.value OR paid IS NULL'
			.$this->sql_order_by();

		$result = $this->db->query($sql);
		$result->setFetchMode(PDO::FETCH_CLASS,$this->model);
		return $result->fetchAll();
	}

	public function fetch_canceled_like($search_model = false, $search = false)
	{
		$models = array();
		$sql = $this->sql_select_all()
			.$this->sql_from()
			.$this->sql_joins('INNER')
			.$this->sql_where_like($search_model, $search, false, 'invoices.status = \'C\'')
			.$this->sql_group_by()
			.$this->sql_order_by();

		$result = $this->db->query($sql);
		$result->setFetchMode(PDO::FETCH_CLASS,$this->model);
		return $result->fetchAll();
	}

	public function emit($search_model = false, $period_time)
	{
		$sql = '
			INSERT INTO '.$this->table.'
				( invoice_id, contract_id, value, discount, date, period_start, period_end, description, comments, status )
			SELECT
				invoices.invoice_id,
				contracts.contract_id,
				services.price AS value,
				0.00 AS discount,
				CONCAT(\''.date('Y-m-',mktime(0,0,0,date('m',$period_time)+1,date('d',$period_time),date('Y',$period_time))).'\',contracts.payday) AS date,
				\''.date('Y-m-',$period_time).'01\' AS period_start,
				\''.date('Y-m-',$period_time).date('t',$period_time).'\' AS period_end,
				CONCAT(\''.lang::cobranca.' '.lang::service.' \',services.name),
				\'\',
				\'A\'
			FROM contracts
			INNER JOIN services ON (
				services.service_id = contracts.service_id )
			LEFT JOIN invoices ON (
				invoices.contract_id = contracts.contract_id
				AND invoices.period_start >= \''.date('Y-m-',$period_time).'01\'
				AND invoices.period_end <= \''.date('Y-m-',$period_time).date('t',$period_time).'\' )'
			.$this->sql_where($search_model,'contracts.date <= \''.date('Y-m-d',$period_time).'\'
				AND (contracts.status = \'L\' OR contracts.status = \'I\' OR contracts.status = \'P\')
				AND services.price > 0').'
			ON DUPLICATE KEY UPDATE
				contract_id = VALUES(contract_id)';
		$this->db->query($sql);

		$sql = '
			INSERT INTO '.$this->table.' (
				invoice_id,
				contract_id,
				value,
				discount,
				date,
				period_start,
				period_end,
				comments,
				status )
			SELECT
				invoices.invoice_id,
				contracts.contract_id,
				( (services.price / '.date('t',$period_time).') * ('.date('t',$period_time).' - (contracts.date - \''.date('Ym',$period_time).'00\') ) ),
				0.00 AS discount,
				CONCAT(\''.date('Y-m-',mktime(0,0,0,date('m',$period_time)+1,date('d',$period_time),date('Y',$period_time))).'\',contracts.payday),
				contracts.date,
				\''.date('Y-m-t',$period_time).'\',
				\'\',
				\'A\'
			FROM contracts
			INNER JOIN services ON (
				services.service_id = contracts.service_id )
			LEFT JOIN invoices ON (
				invoices.contract_id = contracts.contract_id
				AND invoices.period_start = contracts.date
				AND invoices.period_end <= \''.date('Y-m-',$period_time).date('t',$period_time).'\' )'
			.$this->sql_where($search_model,'contracts.date > \''.date('Y-m-d',$period_time).'\'
				AND contracts.date < \''.date('Y-m-t',$period_time).'\'
				AND (contracts.status = \'L\' OR contracts.status = \'I\' OR contracts.status = \'P\')
				AND services.price > 0').'
			ON DUPLICATE KEY UPDATE
				contract_id = VALUES(contract_id),
				period_start = VALUES(period_start)';
		$this->db->query($sql);
	}

}

?>