<?php

/*	Vulpes - Gateway Access Manager
	Copyright © 2000-2002 Pablo Martins F. Costa, João Prado Maia
	Copyright © 200? Daniel William Schultz, Leandro Maniezo
	Copyright © 2006, 2007 Elizeu Alcantara, et al.
	Copyright © 2008 LinFox Serviços em Informatica LTDA. <alwin@linfox.com.br>

	This file is part of Vulpes.

	Vulpes is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Vulpes is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Vulpes.  If not, see <http://www.gnu.org/licenses/>. */

abstract class bank
{
	public $code;
	public $code_dv;
	public $currency_code = '9';

	public function __construct()
	{
		$this->code_dv = $this->modulo_11($this->code);
	}

	protected function get_base_code($invoice)
	{		
		$fator_vencimento = $this->fator_vencimento($invoice->date);
		$net_value = round($invoice->value - $invoice->paid - $invoice->discount, 2);
		
		$code  = number_fit($invoice->account_bank_code,3); // Bank code (3)
		$code .= number_fit($this->currency_code,1);        // Currency code (1)
		$code .= '0';                                       // Digito Verificador - DV (1)
		$code .= number_fit($fator_vencimento,4);           // Payment due factor (4)
		$code .= number_fit($net_value * 100,10);           // Value (10)

		return $code;
	}

	abstract public function get_code($invoice);
	
	/**
	 * Gera uma linha digitavel para uso em boletos.
	 * 
	 * Posição  Conteúdo
	 * 1 a 3    Número do banco
	 * 4        Código da Moeda - 9 para Real
	 * 5        Digito verificador do Código de Barras
	 * 6 a 9    Fator de Vencimento
	 * 10 a 19  Valor (8 inteiros e 2 decimais)
	 * 20 a 44  Campo Livre definido por cada banco (25 caracteres)
	 * 
	 * @param mixed $codigo
	 * @return string
	 */
	function monta_codigo_digitavel($codigo)
	{
		/**
		 * 1. Campo
		 * 
		 * Composto pelo código do banco, código da moéda,
		 * as cinco primeiras posições do campo livre
		 * e DV (modulo10) deste campo
		 */
		$p1 = substr($codigo, 0, 4);
		$p2 = substr($codigo, 19, 5);
		$p3 = $this->modulo_10($p1.$p2);
		$p4 = $p1.$p2.$p3;
		$p5 = substr($p4, 0, 5);
		$p6 = substr($p4, 5);
		$campo1 = $p5.'.'.$p6;
	
		/**
		 * 2. Campo
		 * 
		 * Composto pelas posiçoes 6 a 15 do campo livre 
		 * e livre e DV (modulo10) deste campo
		 */
		$p1 = substr($codigo, 24, 10);
		$p2 = $this->modulo_10($p1);
		$p3 = $p1.$p2;
		$p4 = substr($p3, 0, 5);
		$p5 = substr($p3, 5);
		$campo2 = $p4.'.'.$p5;
	
		/**
		 * 3. Campo
		 * 
		 * Composto pelas posicoes 16 a 25 do campo livre
		 * e livre e DV (modulo10) deste campo
		 */
		$p1 = substr($codigo, 34, 10);
		$p2 = $this->modulo_10($p1);
		$p3 = $p1.$p2;
		$p4 = substr($p3, 0, 5);
		$p5 = substr($p3, 5);
		$campo3 = $p4.'.'.$p5;
	
		/**
		 * 4. Campo
		 * 
		 * Digito verificador do codigo de barras
		 */
		$campo4 = substr($codigo, 4, 1);

		/**
		 * 5. Campo
		 * 
		 * Composto pelo fator vencimento e valor nominal do documento, sem
		 * indicacao de zeros a esquerda e sem edicao (sem ponto e virgula).
		 * Quando se tratar de valor zerado, a representacao deve ser 000
		 * (tres zeros).
		 */
		$p1 = substr($codigo, 5, 4);
		$p2 = substr($codigo, 9, 10);
		$campo5 = $p1.$p2;

		return "$campo1 $campo2 $campo3 $campo4 $campo5";
	}

	function fator_vencimento($date) {
		$date_time = strtotime($date);
		$ref_time = mktime(null,null,null,10,07,1997);
		return abs(($date_time - $ref_time)/86400);
	}

	/**
	 * Calculo do Modulo 10
	 * 
	 * @param string $num
	 * @return int
	 * @author Pablo Martins F. Costa <pablo@users.sourceforge.net>
	 * @copyright Copyright © 2000-2002 Pablo Martins F. Costa
	 */
	function modulo_10($num)
	{ 
		$numtotal10 = 0;
		$fator = 2;

		/* Separacao dos numeros */
		for ($i = strlen($num); $i > 0; $i--) {
			// pega cada numero isoladamente
			$numeros[$i] = substr($num,$i-1,1);
			// Efetua multiplicacao do numero pelo (fator 10)
			// 2002-07-07 01:33:34 Macete para adequar ao Mod10 do Itaú
			$temp = $numeros[$i] * $fator; 
			$temp0 = 0;
			foreach (preg_split('//',$temp,-1,PREG_SPLIT_NO_EMPTY) as $v) {
				$temp0 += $v;
			}
			$parcial10[$i] = $temp0;
			// $numeros[$i] * $fator;
			// monta sequencia para soma dos digitos no (modulo 10)
			$numtotal10 += $parcial10[$i];
			if ($fator == 2) {
				$fator = 1;
			} else {
				$fator = 2; // intercala fator de multiplicacao (modulo 10)
			}
		}

		// várias linhas removidas, vide função original
		// Calculo do modulo 10
		$resto = $numtotal10 % 10;
		$digito = 10 - $resto;
		if ($resto == 0) {
			$digito = 0;
		}

		return $digito;
	}

	/**
	 * Calculo do Modulo 11 para geracao do digito verificador
	 * de boletos bancarios conforme documentos obtidos da
	 * Febraban <www.febraban.org.br>
	 * 
	 * Obs.: Assume-se que a verificação do formato das variáveis
	 * de entrada é feita antes da execução deste script.
	 *
	 * @param string $num String numérica para a qual se deseja calcularo digito verificador
	 * @param int $base Valor maximo de multiplicacao [2-$base]
	 * @param bool $r Quando especificado um devolve somente o resto
	 * @return int Retorna o Digito verificador.
	 * @author Pablo Martins F. Costa <pablo@users.sourceforge.net>
	 * @copyright Copyright © 2000-2002 Pablo Martins F. Costa
	 */
	function modulo_11($num, $base=9, $r=false)
	{
		$soma = 0;
		$fator = 2;

		/* Separacao dos numeros */
		for ($i = strlen($num); $i > 0; $i--) {
			// pega cada numero isoladamente
			$numeros[$i] = substr($num,$i-1,1);
			// Efetua multiplicacao do numero pelo falor
			$parcial[$i] = $numeros[$i] * $fator;
			// Soma dos digitos
			$soma += $parcial[$i];
			if ($fator == $base) {
				// restaura fator de multiplicacao para 2 
				$fator = 1;
			}
			$fator++;
		}

		/* Calculo do modulo 11 */
		if ($r == 0) {
			$soma *= 10;
			$digito = $soma % 11;
			if ($digito == 10) {
				$digito = 0;
			}
			return $digito;
		} elseif ($r == 1) {
			$resto = $soma % 11;
			return $resto;
		}
	}

	function calc_dv($code) {
		$factors = '43290876543298765432987654329876543298765432';
		if (strlen($code) == 44) {
			$sum = 0;
			for ($i=0; $i<44; $i++) {
				$sum += $code[$i] * $factors[$i];
			}
			$num_temp = 11 - ($sum % 11);
			if ($num_temp >= 10) {
				$num_temp = 1;
			}
			$code[4] = $num_temp;
			return $code;
		} else {
			return 'CODE ERROR';
		}
	}
	
}

?>