<?php

/*	Vulpes - Gateway Access Manager
	Copyright © 2007, 2008 LinFox Serviços em Informatica LTDA.

	This file is part of Vulpes.

	Vulpes is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Vulpes is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Vulpes.  If not, see <http://www.gnu.org/licenses/>. */

require_once 'dao/invoice_dao.php';
require_once 'dao/contract_dao.php';

class invoices_plugin extends plugin
{
	public $actions = array(
		'contracts' => array('update','insert','remove'),
		'cron'      => array('update') );

	/**
	 * Call the invoices_plugin
	 * 
	 * @todo When the start/end dates of a contract change, update the invoices.  
	 */
	public function call()
	{
		$config = config::get_instance();
		$db = db::get_instance();

		$invoice_dao = new invoice_dao();
		$contract_dao = new contract_dao();
		$contracts = $contract_dao->fetch_all();

		foreach ($contracts as $contract) {

			if ($contract->invoice_period_end)
				$t_invoice_period_end = strtotime($contract->invoice_period_end); 
			else
				$t_invoice_period_end = 0;
			$t_start = strtotime($contract->start);

			// If the value of the service is 0, no need to bother.
			if ($contract->service_price <= 0) continue;

			if ($contract->service_payment < 1) {

				if ($t_invoice_period_end >= $t_start) continue;
				$t_end = strtotime($contract->end);
				$invoice_value = $contract->service_price;
				
			} else {

				if ($t_invoice_period_end > $t_start)
					$t_start = $t_invoice_period_end;
				$t_next = mktime(
					date('H',$t_start),
					date('i',$t_start),
					date('s',$t_start),
					date('m',$t_start)+1,
					date('d',$t_start),
					date('Y',$t_start) );

				// TODO: add offset to make sure payday isn't too close to next date?
				if (date('d',$t_next) >= $contract->payday) {
					$m_offset = 1;
					$v_offset = 0;
				} else {
					$m_offset = 0;
					$v_offset = 1;
				}

				$t_payday = mktime(
					date('H',$t_next),
					date('i',$t_next),
					date('s',$t_next),
					date('m',$t_next)+$m_offset,
					$contract->payday,
					date('Y',$t_next) );

				$t_end = mktime(
					date('H',$t_payday),
					date('i',$t_payday),
					date('s',$t_payday),
					date('m',$t_payday),
					date('d',$t_payday)-$config->invoice_payday_offset,
					date('Y',$t_payday) );

				if ($t_end > strtotime($contract->end))
					$t_end = strtotime($contract->end);

				$t_value = mktime(
					date('H',$t_next),
					date('i',$t_next),
					date('s',$t_next),
					date('m',$t_next)-$v_offset,
					date('d',$t_next),
					date('Y',$t_next) );
				$invoice_value =
					($contract->service_price / date('t',$t_value))
					* ( ($t_end - $t_start) / 86400);
			}

			if ($t_start >= $t_end) continue;
			if ($t_end > mktime() && $contract->service_payment > 0) continue;

			$invoice = $invoice_dao->fetch_model();
			$invoice->client_id = $contract->client_id;
			$invoice->contract_id = $contract->contract_id;
			$invoice->service_id = $contract->service_id;
			$invoice->method_id = $contract->method_id;
			$invoice->value = $invoice_value;
			$invoice->discount = $contract->discount;
			$invoice->date = date('Y-m-d H:i:s', $t_payday);
			$invoice->period_start = date('Y-m-d H:i:s', $t_start);
			$invoice->period_end = date('Y-m-d H:i:s', $t_end);

			$invoice_dao->insert($invoice);

		}
		return true;
	}
}

?>