/*	Vulpes - Gateway Access Manager
	Copyright © 2007,2008 LinFox Serviços de Informatica LTDA.

	This file is part of Vulpes.

	Vulpes is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Vulpes is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Vulpes.  If not, see <http://www.gnu.org/licenses/>. */

CREATE DATABASE IF NOT EXISTS vulpes
CHARACTER SET utf8 DEFAULT CHARACTER SET utf8
COLLATE utf8_general_ci DEFAULT COLLATE utf8_general_ci;
USE vulpes;

CREATE TABLE IF NOT EXISTS accounts (
	account_id integer NOT NULL auto_increment,
	name varchar(32) NOT NULL,
	bank_name varchar(255) NOT NULL,
	bank_code varchar(3) NOT NULL,
	agency smallint NOT NULL,
	account_number integer NOT NULL,
	carteira smallint NOT NULL,
	convention varchar(8) NOT NULL,	
	currency varchar(4) NOT NULL,
	owner_name varchar(255) NOT NULL,
	owner_cpf_cnpj varchar(20) NOT NULL,
	owner_address varchar(255) NOT NULL,
	owner_city varchar(255) NOT NULL,
	owner_state char(2) NOT NULL,
	payment_number bigint(13) zerofill NOT NULL,
	payment_info1 varchar(80) NOT NULL,
	payment_info2 varchar(80) NOT NULL,
	payment_info3 varchar(80) NOT NULL,
	payment_info4 varchar(80) NOT NULL,
	payment_instructions1 varchar(80) NOT NULL,
	payment_instructions2 varchar(80) NOT NULL,
	payment_instructions3 varchar(80) NOT NULL,
	payment_instructions4 varchar(80) NOT NULL,
	PRIMARY KEY (account_id)
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS brands (
	brand_id integer NOT NULL auto_increment,
	name varchar(32) NOT NULL,
	description varchar(255) NOT NULL,
	PRIMARY KEY (brand_id)
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS config (
	config_id integer NOT NULL auto_increment,
	sitename varchar(32) NOT NULL,
	domain varchar(255) NOT NULL,
	dhcpd_header text NOT NULL,
	iptables text NOT NULL,
	ip varchar(15) NOT NULL,
	cp_port smallint NOT NULL,
	language varchar(5) NOT NULL,
	bandwith varchar(10) NOT NULL,
	iface varchar(10) NOT NULL,
	proxy_port smallint NOT NULL,
	logo_image varchar(255) NOT NULL,
	invoice_type varchar(8) NOT NULL,
	invoice_payday_offset smallint NOT NULL,
	payment_delay smallint NOT NULL,
	log_prune integer NOT NULL,
	template varchar(32) NOT NULL,
	block_overdue boolean NOT NULL,
	cp_header longtext NOT NULL,
	cp_footer longtext NOT NULL,
	PRIMARY KEY (config_id)
) ENGINE=InnoDB;

INSERT INTO config (config_id, sitename, domain, dhcpd_header, iptables, ip, cp_port, language, bandwith, iface, proxy_port, payment_delay, template) VALUES
(1, 'Vulpes', 'localhost', 'shared-network name {\n    option domain-name-servers 10.0.0.1;\n    default-lease-time 600;\n    max-lease-time 7200;\n    option subnet-mask 255.255.255.0;\n\n    subnet 10.0.0.0 netmask 255.255.255.0 {\n        option routers 10.0.0.1;\n    }\n}\n', '# Lista de potas registradas pelo IANA: http://www.iana.org/assignments/port-numbers\n# Lista de portas conhecidas: http://en.wikipedia.org/wiki/List_of_TCP_and_UDP_port_numbers\n#\n# ================================================================ #\n# =============== NETWORK ADDRESS TRANSLATION (NAT) ============== #\n# ================================================================ #\n\n*nat\n:PREROUTING ACCEPT [0:0]\n:POSTROUTING ACCEPT [0:0]\n:OUTPUT ACCEPT [0:0]\n\n{vulpes_nat}\n\n        -A PREROUTING -i <private_iface> -p tcp --dport 80 -j REDIRECT --to-port 82\n        -A POSTROUTING -o <public_iface> -j SNAT --to-source <public_ip>\n\nCOMMIT\n\n\n# ================================================================ #\n# =========================== MANGLE ============================= #\n# ================================================================ #\n*mangle\n:INPUT ACCEPT [0:0]\n:OUTPUT ACCEPT [0:0]\n:FORWARD ACCEPT [0:0]\n:PREROUTING ACCEPT [0:0]\n:POSTROUTING ACCEPT [0:0]\n\n{vulpes_mangle}\n\nCOMMIT\n\n\n# ================================================================ #\n# ============================ FILTER ============================ #\n# ================================================================ #\n*filter\n:INPUT ACCEPT [0:0]\n:OUTPUT ACCEPT [0:0]\n:FORWARD DROP [0:0]\n\n# ================================================ #\n# ======== REGRAS DO FORWARD (ROTEAMENTO) ======== #\n\n        -A FORWARD -m state --state RELATED,ESTABLISHED -j ACCEPT\n        -A FORWARD -d 127.0.0.1 -j ACCEPT\n\n#       ping\n        -A FORWARD -p icmp -j ACCEPT\n\n{vulpes_filter}\n\nCOMMIT\n', '10.0.0.1', 82, 'pt_BR', '10000', 'eth1', 3128, 0, 'xml');


CREATE TABLE IF NOT EXISTS locations (
	location_id integer NOT NULL auto_increment,
	name varchar(32) NOT NULL,
	address1 varchar(255) NOT NULL,
	address2 varchar(255) NOT NULL,
	district varchar(255) NOT NULL,
	postal_code varchar(10) NOT NULL,
	city varchar(255) NOT NULL,
	state varchar(255) NOT NULL,
	country char(2) NOT NULL,
	phone varchar(20) NOT NULL,
	PRIMARY KEY (location_id)
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS networks (
	network_id integer NOT NULL auto_increment,
	iface varchar(8) NOT NULL,
	name varchar(32) NOT NULL,
	network_hex char(8) NOT NULL,
	gateway_hex char(8) NOT NULL,
	broadcast_hex char(8) NOT NULL,
	netmask smallint NOT NULL,
	rate integer NOT NULL,
	rate_default integer NOT NULL,
	PRIMARY KEY (network_id),
	UNIQUE (iface)
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS services (
	service_id integer NOT NULL auto_increment,
	network_id integer NOT NULL,
	name varchar(32) NOT NULL,
	description text NOT NULL,
	price numeric(10,2) NOT NULL,
	price_ip numeric(10,2) NOT NULL,
	payment smallint NOT NULL,
	download integer,
	upload integer,
	auth boolean,
	bind_mac boolean,
	proxy_port integer,
	ttl_limit smallint,
	PRIMARY KEY (service_id)
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS users (
	user_id integer NOT NULL auto_increment,
	username varchar(32) NOT NULL,
	name varchar(255) NOT NULL,
	email varchar(255) NOT NULL,
	language varchar(5) NOT NULL,
	password char(128) NOT NULL,
	admin boolean NOT NULL default false,
	PRIMARY KEY (user_id),
	UNIQUE (username)
) ENGINE=InnoDB;

INSERT INTO users (user_id, username, name, language, password, admin, email) VALUES 
(1, 'admin', 'Administrator', 'en_us', 0x6337616434346362616437363261356461306134353266396538353466646331653065376135326133383031356632336633656162316438306239333164643437323633346466616337316364333465626333356431366162376662386139306338316639373531313364366337353338646336396464386465393037376563, true, 'vulpes@linfox.com.br');


CREATE TABLE IF NOT EXISTS clients (
	client_id integer NOT NULL auto_increment,
	location_id integer NULL,
	username varchar(32) NOT NULL,
	password binary(128) NOT NULL,
	name varchar(255) NOT NULL,
	email varchar(255) NOT NULL,
	address1 varchar(255) NOT NULL,
	address2 varchar(255) NOT NULL,
	district varchar(255) NOT NULL,
	postal_code varchar(10) NOT NULL,
	city varchar(255) NOT NULL,
	state varchar(255) NOT NULL,
	country char(2) NOT NULL,
	phone varchar(21) NOT NULL,
	mobile varchar(21) NOT NULL,
	fiscal_number varchar(15) NOT NULL,
	identity varchar(32) NOT NULL,
	notes text NOT NULL,
	domain varchar(255),
	home_dir varchar(255),
	profile_path varchar(255),
	dir_drive varchar(2),
	logon_script varchar(255),
	creation_time timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
	logon_time integer,
	logoff_time integer,
	kickoff_time integer,
	pass_last_set_time integer,
	pass_can_change_time integer,
	status varchar(32) NOT NULL,
	PRIMARY KEY (client_id),
	FOREIGN KEY (location_id) REFERENCES locations (location_id) ON DELETE SET NULL,
	UNIQUE (username)
) ENGINE=InnoDB;

CREATE TABLE IF NOT EXISTS employees (
	employee_id integer NOT NULL auto_increment,
	location_id integer NULL,
	name varchar(255) NOT NULL,
	email varchar(255) NOT NULL,
	address1 varchar(255) NOT NULL,
	address2 varchar(255) NOT NULL,
	district varchar(255) NOT NULL,
	postal_code varchar(10) NOT NULL,
	city varchar(255) NOT NULL,
	state varchar(255) NOT NULL,
	country char(2) NOT NULL,
	phone varchar(21) NOT NULL,
	mobile varchar(21) NOT NULL,
	fiscal_number varchar(15) NOT NULL,
	identity varchar(32) NOT NULL,
	notes text NOT NULL,
	domain varchar(255),
	home_dir varchar(255),
	profile_path varchar(255),
	dir_drive varchar(2),
	logon_script varchar(255),
	creation_time timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
	logon_time integer,
	logoff_time integer,
	kickoff_time integer,
	pass_last_set_time integer,
	pass_can_change_time integer,
	status varchar(32) NOT NULL,
	PRIMARY KEY (employee_id),
	FOREIGN KEY (location_id) REFERENCES locations (location_id) ON DELETE SET NULL
) ENGINE=InnoDB;

CREATE TABLE IF NOT EXISTS orders (
	order_id integer NOT NULL auto_increment,
	user_id integer NOT NULL,
	porto numeric(10,2) NOT NULL,
	order_date timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
	arrive_date timestamp NOT NULL,
	status varchar(32) NOT NULL,
	PRIMARY KEY (order_id),
	FOREIGN KEY (user_id) REFERENCES users (user_id)
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS products (
	product_id integer NOT NULL auto_increment,
	brand_id integer NOT NULL,
	model varchar(32) NOT NULL,
	description varchar(255) NOT NULL,
	price numeric(10,2) NOT NULL,
	tax integer NOT NULL,
	minimum integer NOT NULL,
	status varchar(32) NOT NULL,
	PRIMARY KEY (product_id),
	FOREIGN KEY (brand_id) REFERENCES brands (brand_id)
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS suppliers (
	supplier_id integer NOT NULL auto_increment,
	location_id integer NULL,
	name varchar(255) NOT NULL,
	company varchar(255) NOT NULL,
	email varchar(64) NOT NULL,
	website varchar(64) NOT NULL,
	registry varchar(16) NOT NULL,
	status varchar(16) NOT NULL,
	PRIMARY KEY (supplier_id),
	FOREIGN KEY (location_id) REFERENCES locations (location_id) ON DELETE SET NULL
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS patrimony (
	patrimony_id integer NOT NULL auto_increment,
	location_id integer NULL,
	product_id integer NOT NULL,
	client_id integer NULL,
	order_id integer NOT NULL,
	parent_id integer NULL,
	name varchar(32) NOT NULL,
	description varchar(255) NOT NULL,
	serial varchar(255) NOT NULL,
	fiscal_number integer NOT NULL,
	mac varchar(255) NULL,
	status varchar(32) NOT NULL,
	PRIMARY KEY (patrimony_id),
	FOREIGN KEY (location_id) REFERENCES locations (location_id) ON DELETE SET NULL,
	FOREIGN KEY (product_id) REFERENCES products (product_id),
	FOREIGN KEY (client_id) REFERENCES clients (client_id),
	FOREIGN KEY (order_id) REFERENCES orders (order_id),
	FOREIGN KEY (parent_id) REFERENCES patrimony (patrimony_id) ON DELETE SET NULL
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS patrimony_locations (
	patrimony_id integer NOT NULL,
	location_id integer NULL,
	start timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
	stop timestamp,
	FOREIGN KEY (patrimony_id) REFERENCES patrimony (patrimony_id) ON UPDATE CASCADE ON DELETE CASCADE,
	FOREIGN KEY (location_id) REFERENCES locations (location_id) ON UPDATE CASCADE ON DELETE CASCADE
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS pcs (
	pc_id integer NOT NULL auto_increment,
	location_id integer NULL,
	group_id integer NOT NULL,
	mac integer NOT NULL,
	status varchar(32) NOT NULL,
	PRIMARY KEY (pc_id),
	FOREIGN KEY (location_id) REFERENCES locations (location_id) ON DELETE SET NULL
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS client_pcs (
	client_id integer NOT NULL,
	pc_id integer NOT NULL,
	FOREIGN KEY (client_id) REFERENCES clients (client_id),
	FOREIGN KEY (pc_id) REFERENCES pcs (pc_id)
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS client_sessions (
	session_id integer NOT NULL auto_increment,
	client_id integer NOT NULL,
	ip varchar(15) NOT NULL,
	mac varchar(17) NOT NULL,
	start timestamp NOT NULL,
	stop timestamp NOT NULL,
	PRIMARY KEY (session_id),
	FOREIGN KEY (client_id) REFERENCES clients (client_id) ON DELETE CASCADE
) ENGINE InnoDB;


CREATE TABLE IF NOT EXISTS tasks (
	task_id integer NOT NULL auto_increment,
	mode varchar(32) NOT NULL,
	action varchar(32) NOT NULL,
	status varchar(32) NOT NULL DEFAULT 'open', 
	PRIMARY KEY (task_id)
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS payment_methods (
	method_id integer NOT NULL auto_increment,
	account_id integer NULL,
	name varchar(32) NOT NULL,
	bill smallint NOT NULL,
	description varchar(255) NOT NULL,
	PRIMARY KEY (method_id),
	FOREIGN KEY (account_id) REFERENCES accounts (account_id)
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS sales (
	sale_id integer NOT NULL auto_increment,
	client_id integer NOT NULL,
	user_id integer NOT NULL,
	date_time timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
	comments text NOT NULL,
	status varchar(32) NOT NULL DEFAULT 'open',
	PRIMARY KEY (sale_id),
	FOREIGN KEY (client_id) REFERENCES clients (client_id),
	FOREIGN KEY (user_id) REFERENCES users (user_id)
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS contracts (
	contract_id integer NOT NULL auto_increment,
	service_id integer NOT NULL,
	client_id integer NOT NULL,
	user_id integer NOT NULL,
	method_id integer NOT NULL,
	start timestamp NOT NULL,
	end timestamp NOT NULL,
	renew boolean NOT NULL,
	discount numeric(10,2) NOT NULL,
	payday smallint NOT NULL,
	status varchar(32) NOT NULL,
	PRIMARY KEY (contract_id),
	FOREIGN KEY (service_id) REFERENCES services (service_id),
	FOREIGN KEY (client_id) REFERENCES clients (client_id),
	FOREIGN KEY (user_id) REFERENCES users (user_id),
	FOREIGN KEY (method_id) REFERENCES payment_methods (method_id)
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS invoices (
	invoice_id integer NOT NULL auto_increment,
	client_id integer NOT NULL,
	contract_id integer NULL,
	service_id integer NULL,
	sale_id integer NULL,
	method_id integer NOT NULL,
	value numeric(10,2) NOT NULL,
	discount numeric(10,2) NOT NULL,
	date date NOT NULL,
	period_start date NOT NULL,
	period_end date NOT NULL,
	description varchar(255) NOT NULL,
	comments text NOT NULL,
	status char(1) NOT NULL,
	PRIMARY KEY (invoice_id),
	FOREIGN KEY (client_id) REFERENCES clients (client_id),
	FOREIGN KEY (contract_id) REFERENCES contracts (contract_id),
	FOREIGN KEY (service_id) REFERENCES services (service_id),
	FOREIGN KEY (sale_id) REFERENCES sales (sale_id),
	FOREIGN KEY (method_id) REFERENCES payment_methods (method_id)
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS log (
	log_id integer NOT NULL auto_increment,
	user_id integer NULL,
	client_id integer NULL,
	mode varchar(32) NOT NULL,
	action varchar(32) NOT NULL,
	date_time timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
	PRIMARY KEY (log_id),
	FOREIGN KEY (user_id) REFERENCES users (user_id) ON DELETE SET NULL,
	FOREIGN KEY (client_id) REFERENCES clients (client_id) ON DELETE SET NULL
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS order_products (
	order_product_id integer NOT NULL auto_increment,
	order_id integer NOT NULL,
	product_id integer NOT NULL,
	supplier_id integer NOT NULL,
	amount integer NOT NULL,
	price numeric(10,2) NOT NULL,
	PRIMARY KEY (order_product_id),
	FOREIGN KEY (order_id) REFERENCES orders (order_id),
	FOREIGN KEY (product_id) REFERENCES products (product_id),
	FOREIGN KEY (supplier_id) REFERENCES suppliers (supplier_id)
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS patrimony_groups (
	group_id integer NOT NULL auto_increment,
	parent_id integer NULL,
	name varchar(32) NOT NULL,
	description varchar(255) NOT NULL,
	PRIMARY KEY (group_id),
	FOREIGN KEY (parent_id) REFERENCES patrimony_groups (group_id) ON DELETE SET NULL
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS patrimony_members (
	member_id integer NOT NULL auto_increment,
	patrimony_id integer NOT NULL,
	group_id integer NOT NULL,
	PRIMARY KEY (member_id),
	FOREIGN KEY (patrimony_id) REFERENCES patrimony (patrimony_id) ON DELETE CASCADE,
	FOREIGN KEY (group_id) REFERENCES patrimony_groups (group_id) ON DELETE CASCADE
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS product_suppliers (
	product_supplier_id integer NOT NULL,
	product_id integer NOT NULL,
	supplier_id integer NOT NULL,
	price float(10,2) NOT NULL,
	status varchar(32) NOT NULL,
	PRIMARY KEY (product_supplier_id),
	UNIQUE (product_id, supplier_id),
    FOREIGN KEY (product_id) REFERENCES products (product_id) ON DELETE CASCADE,
    FOREIGN KEY (supplier_id) REFERENCES suppliers (supplier_id) ON DELETE CASCADE
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS sale_patrimony (
	sale_patrimony_id integer NOT NULL auto_increment,
	sale_id integer NOT NULL,
	patrimony_id integer NOT NULL,
	PRIMARY KEY (sale_patrimony_id),
	UNIQUE (sale_id, patrimony_id),
	FOREIGN KEY (sale_id) REFERENCES sales (sale_id),
	FOREIGN KEY (patrimony_id) REFERENCES patrimony (patrimony_id)
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS schedules (
	schedule_id integer NOT NULL auto_increment,
	service_id integer NOT NULL,
	day varchar(9) NOT NULL,
	start time NOT NULL,
	end time NOT NULL,
	upload integer,
	download integer,
	PRIMARY KEY (schedule_id),
	FOREIGN KEY (service_id) REFERENCES services (service_id) ON DELETE CASCADE
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS tickets (
	ticket_id integer NOT NULL auto_increment,
	user_id integer NULL,
	client_id integer NOT NULL,
	account_id integer NULL,
	summary varchar(255) NOT NULL,
	description longtext NOT NULL,
	type varchar(32) NOT NULL,
	status varchar(32) NOT NULL,
	PRIMARY KEY (ticket_id),
	FOREIGN KEY (user_id) REFERENCES users (user_id) ON DELETE SET NULL,
	FOREIGN KEY (client_id) REFERENCES clients (client_id) ON DELETE CASCADE
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS user_groups (
	group_id integer NOT NULL auto_increment,
	name varchar(32) NOT NULL,
	description varchar(255) NOT NULL,
	PRIMARY KEY (group_id)
) ENGINE=InnoDB;

INSERT INTO user_groups (group_id, name, description)
VALUES (1, 'admin', 'Administrators');


CREATE TABLE IF NOT EXISTS user_members (
	member_id integer NOT NULL auto_increment,
	user_id integer NOT NULL,
	group_id integer NOT NULL,
	PRIMARY KEY (member_id),
	FOREIGN KEY (user_id) REFERENCES users (user_id) ON DELETE CASCADE,
	FOREIGN KEY (group_id) REFERENCES user_groups (group_id)
) ENGINE=InnoDB;

INSERT INTO user_members (user_id, group_id) VALUES (1, 1);


CREATE TABLE IF NOT EXISTS user_permissions (
	permission_id integer NOT NULL auto_increment,
	group_id integer NOT NULL,
	mode varchar(32) NOT NULL,
	action varchar(32) NOT NULL,
	PRIMARY KEY (permission_id),
	FOREIGN KEY (group_id) REFERENCES user_groups (group_id) ON DELETE CASCADE
) ENGINE=InnoDB;

INSERT INTO user_permissions (group_id, mode, action) VALUES (1, '_ALL', '_ALL');


CREATE TABLE IF NOT EXISTS groups (
	group_id integer NOT NULL auto_increment,
	name varchar(32) NOT NULL,
	description varchar(255) NOT NULL,
	PRIMARY KEY (group_id)
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS rules (
	rule_id integer NOT NULL auto_increment,
	group_id integer NOT NULL,
	priority smallint NOT NULL,
	ruletype varchar(32) NOT NULL,
	in_interface varchar(32),
	out_interface varchar(32),
	destination binary(8),
	protocol varchar(32),
	sport smallint,
	dport smallint,
	PRIMARY KEY (rule_id),
	FOREIGN KEY (group_id) REFERENCES groups (group_id)
) ENGINE=InnoDB;

CREATE TABLE IF NOT EXISTS radippool (
	id                    integer NOT NULL auto_increment,
	pool_name             varchar(30) NOT NULL,
	framedipaddress       varchar(15) NOT NULL default '',
	nasipaddress          varchar(15) NOT NULL default '',
	calledstationid       varchar(30) NOT NULL,
	callingstationid      varchar(30) NOT NULL,
	expiry_time           timestamp NULL DEFAULT NULL,
	username              varchar(64) NOT NULL default '',
	pool_key              varchar(30) NOT NULL,
	PRIMARY KEY (id)
) ENGINE=InnoDB;

CREATE TABLE IF NOT EXISTS nas (
	id                    integer NOT NULL auto_increment,
	nasname               varchar(128) NOT NULL,
	shortname             varchar(32),
	type                  varchar(30) DEFAULT 'other',
	ports                 integer,
	secret                varchar(60) NOT NULL DEFAULT 'secret',
	server                varchar(64),
	community             varchar(50),
	description           varchar(200) DEFAULT 'RADIUS Client',
	PRIMARY KEY (id),
	KEY nasname (nasname)
) ENGINE=InnoDB;

CREATE TABLE IF NOT EXISTS radacct (
	radacctid             bigint NOT NULL auto_increment,
	acctsessionid         varchar(32) NOT NULL default '',
	acctuniqueid          varchar(32) NOT NULL default '',
	username              varchar(64) NOT NULL default '',
	groupname             varchar(64) NOT NULL default '',
	realm                 varchar(64) default '',
	nasipaddress          varchar(15) NOT NULL default '',
	nasportid             varchar(15) default NULL,
	nasporttype           varchar(32) default NULL,
	acctstarttime         datetime NULL default NULL,
	acctstoptime          datetime NULL default NULL,
	acctsessiontime       integer default NULL,
	acctauthentic         varchar(32) default NULL,
	connectinfo_start     varchar(50) default NULL,
	connectinfo_stop      varchar(50) default NULL,
	acctinputoctets       bigint default NULL,
	acctoutputoctets      bigint default NULL,
	calledstationid       varchar(50) NOT NULL default '',
	callingstationid      varchar(50) NOT NULL default '',
	acctterminatecause    varchar(32) NOT NULL default '',
	servicetype           varchar(32) default NULL,
	framedprotocol        varchar(32) default NULL,
	framedipaddress       varchar(15) NOT NULL default '',
	acctstartdelay        integer default NULL,
	acctstopdelay         integer default NULL,
	xascendsessionsvrkey  varchar(10) default NULL,
	PRIMARY KEY (radacctid),
	KEY username (username),
	KEY framedipaddress (framedipaddress),
	KEY acctsessionid (acctsessionid),
	KEY acctsessiontime (acctsessiontime),
	KEY acctuniqueid (acctuniqueid),
	KEY acctstarttime (acctstarttime),
	KEY acctstoptime (acctstoptime),
	KEY nasipaddress (nasipaddress)
) ENGINE=InnoDB;

CREATE TABLE IF NOT EXISTS radcheck (
	id integer              unsigned NOT NULL auto_increment,
	username                varchar(64) NOT NULL default '',
	attribute               varchar(32)  NOT NULL default '',
	op                      char(2) NOT NULL DEFAULT '==',
	value                   varchar(253) NOT NULL default '',
	PRIMARY KEY (id),
	KEY username (username(32))
) ENGINE=InnoDB;

CREATE TABLE IF NOT EXISTS radgroupcheck (
	id                      integer unsigned NOT NULL auto_increment,
	groupname               varchar(64) NOT NULL default '',
	attribute               varchar(32)  NOT NULL default '',
	op                      char(2) NOT NULL DEFAULT '==',
	value                   varchar(253)  NOT NULL default '',
	PRIMARY KEY (id),
	KEY groupname (groupname(32))
) ENGINE=InnoDB;

CREATE TABLE IF NOT EXISTS radgroupreply (
	id                      integer unsigned NOT NULL auto_increment,
	groupname               varchar(64) NOT NULL default '',
	attribute               varchar(32)  NOT NULL default '',
	op                      char(2) NOT NULL DEFAULT '=',
	value                   varchar(253) NOT NULL default '',
	PRIMARY KEY (id),
	KEY groupname (groupname(32))
) ENGINE=InnoDB;

CREATE TABLE IF NOT EXISTS radreply (
	id integer              unsigned NOT NULL auto_increment,
	username                varchar(64) NOT NULL default '',
	attribute               varchar(32) NOT NULL default '',
	op                      char(2) NOT NULL DEFAULT '=',
	value                   varchar(253) NOT NULL default '',
	PRIMARY KEY (id),
	KEY username (username(32))
) ENGINE=InnoDB;

CREATE TABLE IF NOT EXISTS radusergroup (
	username                varchar(64) NOT NULL default '',
	groupname               varchar(64) NOT NULL default '',
	priority                integer NOT NULL default '1',
	KEY username (username(32))
) ENGINE=InnoDB;

CREATE TABLE IF NOT EXISTS radpostauth (
	id                      integer NOT NULL auto_increment,
	username                varchar(64) NOT NULL default '',
	pass                    varchar(64) NOT NULL default '',
	reply                   varchar(32) NOT NULL default '',
	authdate                timestamp(14) NOT NULL,
	PRIMARY KEY (id)
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS domains (
	domain_id integer NOT NULL auto_increment,
	domain varchar(255) NOT NULL,
	description varchar(255) NOT NULL,
	aliases integer NOT NULL DEFAULT 0,
	mailboxes integer NOT NULL DEFAULT 0,
	maxquota bigint NOT NULL DEFAULT 0,
	quota bigint NOT NULL DEFAULT 0,
	transport varchar(255) NOT NULL,
	backupmx boolean NOT NULL DEFAULT false,
	created timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
	modified timestamp NOT NULL,
	active boolean NOT NULL DEFAULT true,
	PRIMARY KEY (domain_id),
	UNIQUE KEY (domain)
) ENGINE=InnoDB;

INSERT INTO domains (domain, description, aliases, mailboxes, maxquota, quota, transport, backupmx, created, modified, active)
VALUES ('ALL', '', 0, 0, 0, 0, '', 0, '0000-00-00 00:00:00', '0000-00-00 00:00:00', 1);


CREATE TABLE IF NOT EXISTS alias (
	alias_id integer NOT NULL,
	domain_id integer NOT NULL,
	source varchar(255) NOT NULL,
	destination text NOT NULL,
	active boolean NOT NULL default true,
	PRIMARY KEY (alias_id),
	FOREIGN KEY (domain_id) REFERENCES domains (domain_id)
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS emails (
	email_id integer NOT NULL auto_increment,
	client_id integer NOT NULL,
	domain_id integer NOT NULL,
	email varchar(255) NOT NULL,
	maildir varchar(255) NOT NULL,
	quota bigint NOT NULL default 0,
	active boolean NOT NULL default true,
	PRIMARY KEY (email_id),
	UNIQUE KEY (email),
	FOREIGN KEY (client_id) REFERENCES clients (client_id) ON DELETE CASCADE,
	FOREIGN KEY (domain_id) REFERENCES domains (domain_id)
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS payments (
	payment_id integer NOT NULL auto_increment,
	method_id integer NOT NULL,
	client_id integer NOT NULL,
	user_id integer NOT NULL,
	sale_id integer NULL,
	contract_id integer NULL,
	invoice_id integer NULL,
	value numeric(10,2) NOT NULL,
	interest numeric(10,2) NOT NULL,
	fines numeric(10,2) NOT NULL,
	date date NOT NULL,
	status char(1) NOT NULL,
	PRIMARY KEY (payment_id),
	FOREIGN KEY (method_id) REFERENCES payment_methods (method_id),
	FOREIGN KEY (sale_id) REFERENCES sales (sale_id),
	FOREIGN KEY (contract_id) REFERENCES contracts (contract_id),
	FOREIGN KEY (invoice_id) REFERENCES invoices (invoice_id)
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS client_accounts (
	account_id integer NOT NULL auto_increment,
	client_id integer NOT NULL,
	contract_id integer NULL,
	network_id integer NOT NULL,
	username varchar(32) NOT NULL,
	password varchar(253) NOT NULL,
	mac char(17) NOT NULL,
	ipv4 varchar(15) NOT NULL,
	ipv6 varchar(39) NULL,
	ipv4_hex char(8) NOT NULL,
	ipv6_hex char(32) NULL,
	mac_hex char(12) NOT NULL,
	PRIMARY KEY (account_id),
	FOREIGN KEY (client_id) REFERENCES clients (client_id) ON DELETE CASCADE,
	FOREIGN KEY (contract_id) REFERENCES contracts (contract_id) ON DELETE SET NULL,
	FOREIGN KEY (network_id) REFERENCES networks (network_id),
	UNIQUE (username),
	UNIQUE (mac),
	UNIQUE (ipv4),
	UNIQUE (ipv4_hex),
	UNIQUE (ipv6_hex)
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS members (
	member_id integer NOT NULL auto_increment,
	group_id integer NOT NULL,
	client_id integer NULL,
	account_id integer NULL,
	contract_id integer NULL,
	network_id integer NULL,
	PRIMARY KEY (member_id),
	FOREIGN KEY (group_id) REFERENCES groups (group_id),
	FOREIGN KEY (client_id) REFERENCES clients (client_id) ON DELETE SET NULL,
	FOREIGN KEY (account_id) REFERENCES client_accounts (account_id) ON DELETE SET NULL,
	FOREIGN KEY (contract_id) REFERENCES contracts (contract_id) ON DELETE SET NULL
) ENGINE=InnoDB;